package com.nisum.dice;

import java.util.*;
import java.util.logging.Logger;

/**
 * Player class Implemented from Dice interface
 * and implemented methods are
 * int -roleDice()
 * int -updateScore(String,int)
 * void -startGame();
 */
public class Player implements Dice{
    /**
     * created map object with null value to create single object through getMap() method
     * In map we are going to store player_names and player_scores.
     */

 private static  Map<String,Integer> scoreMap=null;
  private static Random random=null;
  private Logger logger=Logger.getLogger(Dice.class.getName());

    /**
     * Declared old_score variable to assign default score to players
     */
    int oldScore=0;
    /**
     * Declared high_score variable to findOut high score among players
     * and It helps us to announce the winner after scoring 100
     */
    int highScore=0;
    final static int winValue=100;
    char key='r';

    /**
     * roleDice() method gives random number between to 1 to 6 when role the dice
     * @return - int
     */
    @Override
    public int roleDice() {
        if(random==null) {
            random = new Random();
        }
            int num = random.nextInt(6);
            num += 1;
            return num;

    }

    /**
     * getMap() method for create object to the map
     * and same time it will add default player names and scores
     */

   public void getMap(){
       if(scoreMap==null){
           scoreMap=new TreeMap<String,Integer>();
           scoreMap.put("playerName1",oldScore);
           scoreMap.put("playerName2",oldScore);
           scoreMap.put("playerName3",oldScore);
           scoreMap.put("playerName4",oldScore);

       }
   }

    /**
     * updateScore() this method goes to update the score after rolling the dice
     * @param playerName
     * @param addScore
     * @return int
     */
    @Override
    public int updateScore(String playerName,int addScore) {
        int score=0;
        Iterator<Map.Entry<String,Integer>> iterator=scoreMap.entrySet().iterator();
        while(iterator.hasNext()){
            Map.Entry<String,Integer> map=iterator.next();
              if(map.getKey().equals(playerName)){
                   score=map.getValue();
                      score+=addScore;
                      if(score<=winValue) {
                          scoreMap.put(playerName, score);
                      }
                      else{
                          score-=addScore;
                      }
              }
        }
        return score;
    }

    /**
     * showScore() method shows score of each player and it will announce the winner
     */
    public void showScore(){
        String winner="";
        String scoresOfPlayers="";
        Iterator<Map.Entry<String,Integer>> iterator=scoreMap.entrySet().iterator();
        while(iterator.hasNext()){
            Map.Entry<String,Integer> map=iterator.next();
           scoresOfPlayers+= "\n"+map.getKey()+"   :  "+map.getValue();
            if(highScore==winValue && map.getValue()==winValue){
                winner=map.getKey();
            }
        }
        logger.info(scoresOfPlayers);
        if(highScore==winValue){
            logger.info("\n"+winner +" won the game ");
        }

    }

    /**
     * startGame() method it starts the game and this method executes untill one player got 100 score
     * It will scan r character to role dice
     * roleDice() get called to get random number
     * updateScore() get called to update score
     * showScore() get called to show the update score to the players
     */
    @Override
    public void startGame() {
        Scanner sc = new Scanner(System.in);

        while (highScore < winValue) {
            getMap();
            Iterator<Map.Entry<String, Integer>> iterator = scoreMap.entrySet().iterator();
            while (iterator.hasNext()) {
                showScore();
                    Map.Entry<String, Integer> map = iterator.next();
                logger.info("\n"+map.getKey()+" turn role dice-->click r");
                char role = sc.next().charAt(0);
                if (role == key) {
                    int number = roleDice();
                    System.out.println(number);
                    int score= updateScore(map.getKey(), number);
                      if(score>highScore){
                          highScore=score;
                      }
                      else{

                          continue;
                      }
                }
                else{
                    logger.warning("Invalid key");
                    continue;
                }
            }
        }

        showScore();
    }





}
